﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class PlayerManager : MonoBehaviour 
{
	public bool HotSeat = true;
	public Tile StartingTile;
	public string CurrentPlayer;
	public string CurrentPlayer_AP;

	[Header("Assignments")]
	public GameObject player1; //hacker
	public GameObject player2; //PMC 1
	public GameObject player3; //PMC 2
	public GameObject player4; //PMC 3
	private GameObject[] players;

	public GameObject p1Pips;
	public GameObject p2Pips;
	public GameObject p3Pips;
	public GameObject p4Pips;

	[Header("Sprites")]
	public Sprite p1Pips0; public Sprite p1Pips1; public Sprite p1Pips2; public Sprite p1Pips3; public Sprite p1Pips4; 
	public Sprite p1Pips5; 
	public Sprite p2Pips0; public Sprite p2Pips1; public Sprite p2Pips2; public Sprite p2Pips3; 
	public Sprite p3Pips0; public Sprite p3Pips1; public Sprite p3Pips2; public Sprite p3Pips3; 
	public Sprite p4Pips0; public Sprite p4Pips1; public Sprite p4Pips2; public Sprite p4Pips3; 
	public Sprite p1Portrait; public Sprite p2Portrait; public Sprite p3Portrait; public Sprite p4Portrait;
	public GameObject UIPortrait;
	public GameObject winnerDisplay;
	public Sprite hackerWins; public Sprite pmcsWin;

	public static int activePlayer;
	private bool playerCanMove;
	private bool isMoving;
	private Tile currentTile;
	private Tile p1CurrentTile; private Tile p2CurrentTile; private Tile p3CurrentTile; private Tile p4CurrentTile;

	private bool gameIsOver;

	void Start () 
	{
		players = new GameObject[4];
		players [0] = player1; players [1] = player2; players [2] = player3; players [3] = player4;

		CurrentPlayer = "1";
		activePlayer = 1;
		playerCanMove = true;

		gameIsOver = false;

		UIPortrait.GetComponent<Image>().sprite = p1Portrait;
		winnerDisplay.SetActive (false);

		GetCurrentTile ();
	}
	
	// Update is called once per frame
	void Update () 
	{
		InputCheck ();

		if(gameIsOver && inp_Confirm)
		{
			Application.LoadLevel (0);
		}
		else if (!gameIsOver)
		{
			//Check to see if the current player has moves left
			if((activePlayer == 1 && GameManager.P1_AP > 0) || (activePlayer == 2 && GameManager.P2_AP > 0) 
				|| (activePlayer == 3 && GameManager.P3_AP > 0) || (activePlayer == 4 && GameManager.P4_AP > 0))
			{
				playerCanMove = true;
				GameManager.noMoreMoves = false;
			}
			else 
			{
				if (GameManager.noMoreMoves == false)
				{
					p1Pips.SetActive (false);
					p2Pips.SetActive (false);
					p3Pips.SetActive (false);
					p4Pips.SetActive (false);
					playerCanMove = false;
					GameManager.noMoreMoves = true;
				}
			}

			//Check which player is active and display the move counter appropriately
			if(playerCanMove)
			{
				SwapPlayerVisuals();
			}
			
			if(playerCanMove == true && isMoving == false)
			{
				if(inp_Up)
				{
					AttemptMove(TileDirection.Up);
				}
				else if(inp_Down && currentTile.TileCoords.y != 0)
				{
					AttemptMove(TileDirection.Down);
				}
				else if (inp_Left)
				{
					AttemptMove(TileDirection.Left);
				}
				else if (inp_Right)
				{
					AttemptMove(TileDirection.Right);
				}
			}

			if(inp_Confirm)
			{
				//Check to see if p1 is in control and/or stuck on top of a PMC
				if(activePlayer == 1)
				{
					if(currentTile == p2CurrentTile || currentTile == p3CurrentTile || currentTile == p4CurrentTile)
					{
						//Display "Unable to end turn" message!
						GameOver("pmc");
					}
					else
					{
						ChangeTurns();
					}
				}
				else
				{
					ChangeTurns ();
				}
			}
		}
	}

	void AttemptMove(TileDirection dir)
	{
		bool moveValid = false;

		switch(dir)
		{
		case (TileDirection.Up):
		{
			isMoving = true;
			
			//Check against map matrix to make sure this move is possible; if so...
			if(currentTile.NeighborDirections.Contains(TileDirection.Up))
			{
				moveValid = true;
			}
			else if (currentTile.PathDirections.Contains(TileDirection.Up))
			{
				moveValid = true;
				TileManager.instance.GenerateTile(currentTile.TileCoords+TileManager.vectors[2], currentTile);
			}
			else
				moveValid = false;
			
			if(moveValid)
			{
				MoveUp();
				ReduceAP();
				
				GetCurrentTile();
			}
			
			isMoving = false;
		}
			break;
		case (TileDirection.Down):
		{
			isMoving = true;
			//Check against map matrix to make sure this move is possible; if so...
			if(currentTile.NeighborDirections.Contains(TileDirection.Down))
			{
				moveValid = true;
			}
			else if (currentTile.PathDirections.Contains(TileDirection.Down))
			{
				moveValid = true;
				TileManager.instance.GenerateTile(currentTile.TileCoords+TileManager.vectors[0], currentTile);
			}
			else
				moveValid = false;
			
			if(moveValid)
			{
				MoveDown();
				ReduceAP();
				
				GetCurrentTile ();
			}
			
			isMoving = false;
		}
			break;
		case (TileDirection.Left):
		{
			isMoving = true;
			//Check against map matrix to make sure this move is possible; if so...
			if(currentTile.NeighborDirections.Contains(TileDirection.Left))
			{
				moveValid = true;
			}
			else if (currentTile.PathDirections.Contains(TileDirection.Left))
			{
				moveValid = true;
				TileManager.instance.GenerateTile(currentTile.TileCoords+TileManager.vectors[1], currentTile);
			}
			else
				moveValid = false;
			
			if(moveValid)
			{
				MoveLeft();
				ReduceAP();
				
				GetCurrentTile ();
			}
			
			isMoving = false;
		}
			break;
		case (TileDirection.Right):
		{
			isMoving = true;
			//Check against map matrix to make sure this move is possible; if so...
			if(currentTile.NeighborDirections.Contains(TileDirection.Right))
			{
				moveValid = true;
			}
			else if (currentTile.PathDirections.Contains(TileDirection.Right))
			{
				moveValid = true;
				TileManager.instance.GenerateTile(currentTile.TileCoords+TileManager.vectors[3], currentTile);
			}
			else
				moveValid = false;
			
			if(moveValid)
			{
				MoveRight();
				ReduceAP();
				
				GetCurrentTile ();
			}
			
			isMoving = false;
		}
			break;
		}
	}

	void ReduceAP()
	{
		if(activePlayer == 1)
		{
			GameManager.P1_AP -= 1;
			CurrentPlayer_AP = GameManager.P1_AP.ToString();
		}
		else if(activePlayer == 2)
		{
			GameManager.P2_AP -= 1;
			CurrentPlayer_AP = GameManager.P2_AP.ToString();
		}
		else if(activePlayer == 3)
		{
			GameManager.P3_AP -= 1;
			CurrentPlayer_AP = GameManager.P3_AP.ToString();
		}
		else if (activePlayer == 4)
		{
			GameManager.P4_AP -= 1;
			CurrentPlayer_AP = GameManager.P4_AP.ToString();
		}
	}

	void GetCurrentTile()
	{
		RaycastHit hit;
		Vector3 tempVect = players[activePlayer-1].transform.position;
		tempVect.z = tempVect.z-1;
		if (Physics.Raycast(tempVect, Vector3.forward*2f, out hit))
		{
			currentTile = hit.collider.gameObject.GetComponent<Tile>();
		}
	}

	void ChangeTurns()
	{
		GetCurrentTile ();

		//Save last tile of this player
		if (activePlayer == 1) {
			p1CurrentTile = currentTile;
			if(p1CurrentTile.TileCoords == GameManager.instance.MainframeCoords)
				GameOver("hacker");
		}
		else if (activePlayer == 2)
		{
			p2CurrentTile = currentTile;
			if(p2CurrentTile == p1CurrentTile)
				GameOver("pmc");
		}
		else if (activePlayer == 3)
		{
			p3CurrentTile = currentTile;
			if(p3CurrentTile == p1CurrentTile)
				GameOver("pmc");
		}
		else if (activePlayer == 4)
		{
			p4CurrentTile = currentTile;
			if(p4CurrentTile == p1CurrentTile)
				GameOver("pmc");
		}

		//Set active player to the next one in order
		if(activePlayer != 4)
		{
			activePlayer++;


		}
		else if (activePlayer == 4)
		{
			activePlayer = 1;
			GameManager.instance.RoundOver();
		}

		//Change the portrait accordingly
		if (activePlayer == 1) {
			UIPortrait.GetComponent<Image> ().sprite = p1Portrait;
		}
		else if (activePlayer == 2) {
			UIPortrait.GetComponent<Image>().sprite = p2Portrait;
		}
		else if (activePlayer == 3) {
			UIPortrait.GetComponent<Image>().sprite = p3Portrait;
		}
		else if (activePlayer == 4) {
			UIPortrait.GetComponent<Image>().sprite = p4Portrait;
		}

		CurrentPlayer = activePlayer.ToString ();

		GameManager.noMoreMoves = false;
		GameManager.instance.ResetAP();

		SoundManager.instance.TurnChange ();
	}

	void GameOver(string winner)
	{
		gameIsOver = true;
		if (winner == "pmc")
		{
			//Show PMCs Win screen
			winnerDisplay.SetActive(true);
			winnerDisplay.GetComponent<SpriteRenderer>().sprite = pmcsWin;
			SoundManager.instance.WinCorp ();


		}
		else if (winner == "hacker")
		{
			//Show Hacker Wins screen
			winnerDisplay.SetActive (true);
			winnerDisplay.GetComponent<SpriteRenderer>().sprite = hackerWins;
			SoundManager.instance.WinHacker ();
		}
	}

	void SwapPlayerVisuals()
	{
		if(activePlayer == 1)
		{
			//Show pips over p1's head
			p1Pips.SetActive (true);
			p2Pips.SetActive (false);
			p3Pips.SetActive (false);
			p4Pips.SetActive (false);
			if(GameManager.P1_AP == 5)
			{
				p1Pips.GetComponent<SpriteRenderer>().sprite = p1Pips5;
			}
			else if(GameManager.P1_AP == 4)
			{
				p1Pips.GetComponent<SpriteRenderer>().sprite = p1Pips4;
			}
			else if(GameManager.P1_AP == 3)
			{
				p1Pips.GetComponent<SpriteRenderer>().sprite = p1Pips3;
			}
			else if(GameManager.P1_AP == 2)
			{
				p1Pips.GetComponent<SpriteRenderer>().sprite = p1Pips2;
			}
			else if(GameManager.P1_AP == 1)
			{
				p1Pips.GetComponent<SpriteRenderer>().sprite = p1Pips1;
			}
			else if(GameManager.P1_AP == 0)
			{
				p1Pips.GetComponent<SpriteRenderer>().sprite = p1Pips0;
			}
		}
		else if(activePlayer == 2)
		{
			//Show pips over p2's head
			p1Pips.SetActive (false);
			p2Pips.SetActive (true);
			p3Pips.SetActive (false);
			p4Pips.SetActive (false);
			//Show correct number of pips
			if(GameManager.P2_AP == 3)
			{
				p2Pips.GetComponent<SpriteRenderer>().sprite = p2Pips3;
			}
			else if(GameManager.P2_AP == 2)
			{
				p2Pips.GetComponent<SpriteRenderer>().sprite = p2Pips2;
			}
			else if(GameManager.P2_AP == 1)
			{
				p2Pips.GetComponent<SpriteRenderer>().sprite = p2Pips1;
			}
			else if(GameManager.P2_AP == 0)
			{
				p2Pips.GetComponent<SpriteRenderer>().sprite = p2Pips0;
			}
		}
		else if(activePlayer == 3)
		{
			//Show pips over p3's head
			p1Pips.SetActive (false);
			p2Pips.SetActive (false);
			p3Pips.SetActive (true);
			p4Pips.SetActive (false);
			//Show correct number of pips
			if(GameManager.P3_AP == 3)
			{
				p3Pips.GetComponent<SpriteRenderer>().sprite = p2Pips3;
			}
			else if(GameManager.P3_AP == 2)
			{
				p3Pips.GetComponent<SpriteRenderer>().sprite = p2Pips2;
			}
			else if(GameManager.P3_AP == 1)
			{
				p3Pips.GetComponent<SpriteRenderer>().sprite = p2Pips1;
			}
			else if(GameManager.P3_AP == 0)
			{
				p3Pips.GetComponent<SpriteRenderer>().sprite = p2Pips0;
			}
		}
		else if(activePlayer == 4)
		{
			//Show pips over p4's head
			p1Pips.SetActive (false);
			p2Pips.SetActive (false);
			p3Pips.SetActive (false);
			p4Pips.SetActive (true);
			//Show correct number of pips
			if(GameManager.P4_AP == 3)
			{
				p4Pips.GetComponent<SpriteRenderer>().sprite = p2Pips3;
			}
			else if(GameManager.P4_AP == 2)
			{
				p4Pips.GetComponent<SpriteRenderer>().sprite = p2Pips2;
			}
			else if(GameManager.P4_AP == 1)
			{
				p4Pips.GetComponent<SpriteRenderer>().sprite = p2Pips1;
			}
			else if(GameManager.P4_AP == 0)
			{
				p4Pips.GetComponent<SpriteRenderer>().sprite = p2Pips0;
			}
		}
	}

	void MoveUp()
	{
		if(activePlayer == 1)
		{
			player1.transform.position += new Vector3(0, 1, 0);
			//Subtract one tick from the player's available moves
			isMoving = false;
			SoundManager.instance.MoveHacker ();
		}
		else if(activePlayer == 2)
		{
			player2.transform.position += new Vector3(0, 1, 0);
			//Subtract one tick from the player's available moves
			isMoving = false;
			SoundManager.instance.MoveCorp ();
		}
		else if(activePlayer == 3)
		{
			player3.transform.position += new Vector3(0, 1, 0);
			//Subtract one tick from the player's available moves
			isMoving = false;
			SoundManager.instance.MoveCorp ();
		}
		else if(activePlayer == 4)
		{
			player4.transform.position += new Vector3(0, 1, 0);
			//Subtract one tick from the player's available moves
			isMoving = false;
			SoundManager.instance.MoveCorp ();
		}
	}
	void MoveDown()
	{
		if(activePlayer == 1)
		{
			player1.transform.position += new Vector3(0, -1, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveHacker ();
		}
		else if(activePlayer == 2)
		{
			player2.transform.position += new Vector3(0, -1, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
		else if(activePlayer == 3)
		{
			player3.transform.position += new Vector3(0, -1, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
		else if(activePlayer == 4)
		{
			player4.transform.position += new Vector3(0, -1, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
	}
	void MoveRight()
	{
		if(activePlayer == 1)
		{
			player1.transform.position += new Vector3(1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveHacker ();
		}
		else if(activePlayer == 2)
		{
			player2.transform.position += new Vector3(1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
		else if(activePlayer == 3)
		{
			player3.transform.position += new Vector3(1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
		else if(activePlayer == 4)
		{
			player4.transform.position += new Vector3(1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
	}
	void MoveLeft()
	{
		if(activePlayer == 1)
		{
			player1.transform.position += new Vector3(-1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveHacker ();
		}
		else if(activePlayer == 2)
		{
			player2.transform.position += new Vector3(-1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
		else if(activePlayer == 3)
		{
			player3.transform.position += new Vector3(-1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
		else if(activePlayer == 4)
		{
			player4.transform.position += new Vector3(-1, 0, 0);
			//Subtract one tick from the player's available moves
			SoundManager.instance.MoveCorp ();

		}
	}

	bool inp_Up;
	bool inp_Down;
	bool inp_Left;
	bool inp_Right;
	bool inp_Confirm;
	bool inp_Deny;
	float inp_DeadZ = 0.3f;

	void InputCheck()
	{
		if(Input.GetKeyDown ("up") || Input.GetKeyDown ("w"))
		{
			inp_Up = true;
		}
		else
			inp_Up = false;

		if(Input.GetKeyDown ("down") || Input.GetKeyDown ("s"))
		{
			inp_Down = true;
		}
		else
			inp_Down = false;

		if(Input.GetKeyDown ("left") || Input.GetKeyDown ("a"))
		{
			inp_Left = true;
		}
		else
			inp_Left = false;

		if(Input.GetKeyDown ("right") || Input.GetKeyDown ("d"))
		{
			inp_Right = true;
		}
		else
			inp_Right = false;

		if(Input.GetButtonDown ("P1_Confirm") || Input.GetKeyDown ("enter") || Input.GetKeyDown ("return"))
		{
			inp_Confirm = true;
		}
		else
			inp_Confirm = false;

		if(Input.GetButtonDown ("P1_Deny"))
		{
			inp_Deny = true;
		}
		else
			inp_Deny = false;
	}
}
