using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class TileManager : MonoBehaviour 
{
	public static TileManager instance;

	public Tile TilePrefab;
	public Transform Parent;

	public static List<Tile> Tiles;
	public List<Tile> LobbyTiles = new List<Tile>();
	public static List<Vector2> TileCoords;

	public Sprite tilespriteTerminal;
	public Sprite tilespriteMainframe;
	public Sprite tilespriteCarpet1;
	public Sprite tilespriteCarpet2;
	public Sprite tilespriteCarpet3;
	public Sprite tilespriteCarpet4;
	public Sprite tilespriteCarpet5;

	public static Vector2[] vectors = 
	{
		new Vector2 (0, -1),
		new Vector2 (-1, 0),
		new Vector2 (0, 1),
		new Vector2 (1, 0)
	};

	void Start () 
	{
		instance = this;

		Tiles = new List<Tile> ();
		foreach(Tile lob in LobbyTiles)
			Tiles.Add (lob);
		TileCoords = new List<Vector2> {
			new Vector2 (0, 0),
			new Vector2 (1, 0),
			new Vector2 (-1, 0),
			new Vector2 (0, 1),
			new Vector2 (1, 1),
			new Vector2 (-1, 1),
		};
	}

	void Update () 
	{
	
	}

	public void GenerateTile(Vector2 tileCoord, Tile tilePrevious)
	{
//		if(tileCoords.Contains(tileCoord) == false && tileCoord.y > 0) 
		//validity check, goes to player code
		List<TileDirection> pathCandidateDirections = new List<TileDirection>();

		Tile tile = Instantiate(TilePrefab, tileCoord, transform.rotation) as Tile;
		tile.transform.parent = Parent;
		tile.gameObject.name = "Tile - " + tileCoord;
		tile.TileCoords = tileCoord;

		for(int hoge = 0; hoge < 4; hoge++) // iterate through potential neighbors
		{
			switch(hoge)
			{
			case (0):
			{
				if(tile.TileCoords.y != 0)
				{
					Tile tileToCheck = null;
					foreach(Tile check in Tiles) // check for existing tile
					{
						if(check.TileCoords == tileCoord + vectors[0])
							tileToCheck = check;
					}
					if(tileToCheck != null)
					{
						if( !(tile.Neighbors.Contains(tileToCheck)) && tileToCheck.PathDirections.Contains(TileDirection.Up) )
						{
							tile.Neighbors.Add(tileToCheck);
							tile.NeighborDirections.Add(TileDirection.Down);
						}
					}
					else //no neighbor, check for possible moves
					{
						pathCandidateDirections.Add(TileDirection.Down);
					}
				}
			}
				break;
			case (1):
			{
				Tile tileToCheck = null;

				foreach(Tile check in Tiles) // check for existing tile
				{
					if(check.TileCoords == tileCoord + vectors[1])
						tileToCheck = check;
				}
				if(tileToCheck != null)
				{
					if( !(tile.Neighbors.Contains(tileToCheck)) && tileToCheck.PathDirections.Contains(TileDirection.Right) )
					{
						tile.Neighbors.Add(tileToCheck);
						tile.NeighborDirections.Add(TileDirection.Left);
					}
				}
				else //no neighbor, check for possible moves
				{
					pathCandidateDirections.Add(TileDirection.Left);
				}
			}
				break;
			case (2):
			{
				Tile tileToCheck = null;
				foreach(Tile check in Tiles) // check for existing tile
				{
					if(check.TileCoords == tileCoord + vectors[2])
						tileToCheck = check;
				}
				if(tileToCheck != null)
				{
					if( !(tile.Neighbors.Contains(tileToCheck)) && tileToCheck.PathDirections.Contains(TileDirection.Down) )
					{
						tile.Neighbors.Add(tileToCheck);
						tile.NeighborDirections.Add(TileDirection.Up);
					}
				}
				else //no neighbor, check for possible moves
				{
					pathCandidateDirections.Add(TileDirection.Up);
				}
			}
				break;
			case (3):
			{
				Tile tileToCheck = null;
				foreach(Tile check in Tiles) // check for existing tile
				{
					if(check.TileCoords == tileCoord + vectors[3])
						tileToCheck = check;
				}
				if(tileToCheck != null)
				{
					if( !(tile.Neighbors.Contains(tileToCheck)) && tileToCheck.PathDirections.Contains(TileDirection.Left) )
					{
						tile.Neighbors.Add(tileToCheck);
						tile.NeighborDirections.Add(TileDirection.Right);
					}
				}
				else //no neighbor, check for possible moves
				{
					pathCandidateDirections.Add(TileDirection.Right);
				}
			}
				break;
			}
		}

		Vector2 coordDiff = tileCoord - tilePrevious.TileCoords;
		TileDirection newDir = TileDirection.Down;
		if(coordDiff == vectors[0])
			newDir = TileDirection.Down;
		else if(coordDiff == vectors[1])
			newDir =TileDirection.Left;
		else if(coordDiff == vectors[2])
			newDir =TileDirection.Up;
		else if(coordDiff == vectors[3])
			newDir =TileDirection.Right;

		tilePrevious.PathDirections.Remove(newDir);
		tilePrevious.Neighbors.Add(tile);
		tilePrevious.NeighborDirections.Add(newDir);

		// check for open paths 
		for(int hoge = 1; hoge < pathCandidateDirections.Count; hoge++) 
		{
			switch(tile.Doors)	
			{
			case(1):
			{
				int randoChance = Random.Range(0,100);
				if (randoChance < GameManager.instance.Probability_SecondDoor)
				{
					tile.PathDirections.Add(pathCandidateDirections[hoge]);
					tile.Doors++;

					DoorManager.SpawnDoor(tile.gameObject, pathCandidateDirections[hoge]);
				}
			}
				break;
			case(2):
			{
				int randoChance = Random.Range(0,100);
				if (randoChance < GameManager.instance.Probability_ThirdDoor)
				{
					tile.PathDirections.Add(pathCandidateDirections[hoge]);
					tile.Doors++;
					DoorManager.SpawnDoor(tile.gameObject, pathCandidateDirections[hoge]);
				}
			}
				break;
			case(3):
			{
				int randoChance = Random.Range(0,100);
				if (randoChance < GameManager.instance.Probability_FourthDoor)
				{
					tile.PathDirections.Add(pathCandidateDirections[hoge]);
					tile.Doors++;
					DoorManager.SpawnDoor(tile.gameObject, pathCandidateDirections[hoge]);
				}
			}
				break;
			}
		}

		for(int hoge = 0; hoge < 4; hoge++) 
		{
			switch(hoge)
			{
			case(0):
			{
				if( !(tile.NeighborDirections.Contains(TileDirection.Down)) && !(tile.PathDirections.Contains(TileDirection.Down)))
					DoorManager.SpawnWall(tile.gameObject, TileDirection.Down);
			}
				break;
			case(1):
			{
				if( !(tile.NeighborDirections.Contains(TileDirection.Left)) && !(tile.PathDirections.Contains(TileDirection.Left)))
					DoorManager.SpawnWall(tile.gameObject, TileDirection.Left);
			}
				break;
			case(2):
			{
				if( !(tile.NeighborDirections.Contains(TileDirection.Up)) && !(tile.PathDirections.Contains(TileDirection.Up)))
					DoorManager.SpawnWall(tile.gameObject, TileDirection.Up);
			}
				break;
			case(3):
			{
				if( !(tile.NeighborDirections.Contains(TileDirection.Right)) && !(tile.PathDirections.Contains(TileDirection.Right)))
					DoorManager.SpawnWall(tile.gameObject, TileDirection.Right);
			}
				break;
			}
		}

		if(GameManager.MainframeInPlay = true)
		{
			int mfChance = Random.Range(0,100);
			if (mfChance < GameManager.instance.Probability_Mainframe)
			{
				tile.IsMainframe = true;
				GameManager.MainframeInPlay = true;
				GameManager.instance.Mainframe = "MAINFRAME";
				GameManager.instance.MainframeCoords = tile.TileCoords;
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteMainframe;
			}
		}

		if(tile.IsMainframe == false)
		{
			int termChance = Random.Range(0,100);
			if (termChance < GameManager.instance.Probability_Terminal)
			{
				tile.IsTerminal = true;
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteTerminal;
			}
		}

		if(tile.IsMainframe == false && tile.IsTerminal == false)
		{
			int randoTileCarpet = Random.Range(0,5);
			switch(randoTileCarpet)
			{
			case(0):
			{
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteCarpet1;
			}
				break;
			case(1):
			{
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteCarpet2;
			}
				break;
			case(2):
			{
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteCarpet3;
			}
				break;
			case(3):
			{
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteCarpet4;
			}
				break;
			case(4):
			{
				tile.GetComponent<SpriteRenderer>().sprite = tilespriteCarpet5;
			}
				break;
			}
		}

		TileCoords.Add(tileCoord);
		Tiles.Add (tile);
	}
}